//+------------------------------------------------------------------+
//|                                               ADX_with_Alert.mq4 |
//|                                   Copyright  2008, Serega Lykov |
//|                                       http://mtexperts.narod.ru/ |
//+------------------------------------------------------------------+

#property copyright "Copyright  2008, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

//---- property of indicator ----------------------------------------+
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 LightSeaGreen
#property indicator_color2 YellowGreen
#property indicator_color3 Wheat

//---- external parameters ------------------------------------------+
extern string _____ADX______________;
extern int    ADX_Period          = 14;       // the period of ADX
extern int    ADX_AppPrice        = 0;        // price for ADX:     0 = PRICE_CLOSE, 1 = PRICE_OPEN,
                                              // 2 = PRICE_HIGH,    3 = PRICE_LOW,   4 = 0PRICE_MEDIAN,
                                              // 5 = PRICE_TYPICAL, 6 = PRICE_WEIGHTED
extern bool   SignalOnlyAfterCloseBar = false; // true = check signal only after closing bar
extern bool   UseSmooth_ADX       = false;    // true = use smoothing of value ADX
extern int    ADX_MASmooth_Period = 7;        // the period of smoothing MA (for ADX)
extern int    ADX_MASmooth_Method = 0;        // the method of smoothing MA (for ADX): 0-Simple, 1-Exponential, 2-Smoothed, 3-Weighted
extern bool   UseSmooth_DI        = false;    // true = use smoothing of values DI
extern int    DI_MASmooth_Period  = 7;        // the period of smoothing MA (for DI)
extern int    DI_MASmooth_Method  = 0;        // the method of smoothing MA (for DI): 0-Simple, 1-Exponential, 2-Smoothed, 3-Weighted
extern string _____Signal_ADX_______;
extern bool   UseSignal_ADX       = true;     // true = alerts at signal from ADX
extern int    Level               = 20;       // value of level for signal from ADX
extern color  LevelColor          = RoyalBlue; // the color of level
extern bool   ADX_EmailAlert      = true;     // true = alert to e-mail
extern bool   ADX_PopUpAlert      = true;     // true = alert to PopUp-window
extern bool   ADX_SoundAlert      = true;     // true = play sound-file at signal
extern string ADX_SoundFile_Buy   = "alert.wav";  // the name sound-file at Buy-signal
extern string ADX_SoundFile_Sell  = "alert2.wav"; // the name sound-file at Sell-signal
extern bool   ADX_DrawVertLine    = true;     // true = draw vertical line at signal-bar
extern color  ADX_VLineColor_Buy  = Blue;     // the color of vertial line at Buy-signal
extern color  ADX_VLineColor_Sell = Red;      // the color of vertial line at Sell-signal
extern string _____Signal_DI________;
extern bool   UseSignal_DI        = true;     // true = alerts at signal from DI
extern bool   DI_EmailAlert       = true;     // true = alert to e-mail
extern bool   DI_PopUpAlert       = true;     // true = alert to PopUp-window
extern bool   DI_SoundAlert       = true;     // true = play sound-file at signal
extern string DI_SoundFile_Buy    = "alert.wav";  // the name sound-file at Buy-signal
extern string DI_SoundFile_Sell   = "alert2.wav"; // the name sound-file at Sell-signal
extern bool   DI_DrawVertLine     = true;     // true = draw vertical line at signal-bar
extern color  DI_VLineColor_Buy   = Blue;     // the color of vertial line at Buy-signal
extern color  DI_VLineColor_Sell  = Red;      // the color of vertial line at Sell-signal

//---- buffers ------------------------------------------------------+
static double ADXBuffer[];
static double pDIBuffer[];
static double mDIBuffer[];
static double ADXBufferSmooth[];
static double pDIBufferSmooth[];
static double mDIBufferSmooth[];

//---- global variables ---------------------------------------------+
static datetime prevtime_adx;
static datetime prevtime_di;
static string   name_level  = "adx_hline";
static string   adx_message = "The line ADX to cross a Level from below to top.";
static string   pdi_message = "The line +DI to cross a line -DI from below to top.";
static string   mdi_message = "The line -DI to cross a line +DI from below to top.";
static string   short_name;
static int      shift;

//---- initialization of indicator ----------------------------------+
int init()
  {
//---- the 3 additional buffer
   IndicatorBuffers(6);
//---- set a "short" name of the indicator
   if(!UseSmooth_ADX && !UseSmooth_DI) string smooth = "";
   else
     {
      if(!UseSmooth_ADX) string smt_adx = "n/u";
      else
        {
         switch(ADX_MASmooth_Method)
           {
            case 1: { smt_adx = "EMA(";  break; }
            case 2: { smt_adx = "SMMA("; break; }
            case 3: { smt_adx = "LWMA("; break; }
            default:  smt_adx = "SMA(";
           }
         smt_adx = StringConcatenate(smt_adx,ADX_MASmooth_Period,")");
        }
      if(!UseSmooth_DI) string smt_di = "n/u";
      else
        {
         switch(DI_MASmooth_Method)
           {
            case 1: { smt_di = "EMA(";  break; }
            case 2: { smt_di = "SMMA("; break; }
            case 3: { smt_di = "LWMA("; break; }
            default:  smt_di = "SMA(";
           }
         smt_di = StringConcatenate(smt_di,DI_MASmooth_Period,")");
        }
      smooth = StringConcatenate(" with Smooth [",smt_adx,",",smt_di,"]");
     }
   short_name = StringConcatenate("ADX_with_Alert(",ADX_Period,")",smooth);
   IndicatorShortName(short_name);
//---- set a accuracy of values of the indicator
   IndicatorDigits(4);
//---- set a style for line
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID);
   SetIndexStyle(1,DRAW_LINE,STYLE_DOT);
   SetIndexStyle(2,DRAW_LINE,STYLE_DOT);
//---- set a arrays for line
   SetIndexBuffer(0,ADXBuffer);
   SetIndexBuffer(1,pDIBuffer);
   SetIndexBuffer(2,mDIBuffer);
   SetIndexBuffer(3,ADXBufferSmooth);
   SetIndexBuffer(4,pDIBufferSmooth);
   SetIndexBuffer(5,mDIBufferSmooth);
//---- set a first bar for drawing the line
   SetIndexDrawBegin(0,ADX_Period);
   SetIndexDrawBegin(1,ADX_Period);
   SetIndexDrawBegin(2,ADX_Period);
//---- set a names for lines
   SetIndexLabel(0,"ADX");
   SetIndexLabel(1,"+DI");
   SetIndexLabel(2,"-DI");
//---- set a constant
   if(SignalOnlyAfterCloseBar) shift = 1;
   else shift = 0;
//---- initialization of parameters
   prevtime_adx = Time[0];
   prevtime_di = Time[0];
//---- finish of initialization
   return(0);
  }

//---- deinitialization of indicator --------------------------------+
int deinit()
  {
//---- delete objects of indicator
   int obj_total = ObjectsTotal() - 1;
   for(int i=obj_total; i>=0; i--)
     {
      string name = ObjectName(i);
      if(ObjectType(name) != OBJ_VLINE) continue;
      if(StringFind(name,"adx_vline",0) < 0) continue;
      ObjectDelete(name);
     }
   return(0);
  }

//---- ADX with Alert -----------------------------------------------+
int start()
  {
   // draw level
   if(ObjectFind(name_level) < 0)
     {
      ObjectCreate(name_level,OBJ_HLINE,WindowFind(short_name),0,Level);
      ObjectSet(name_level,OBJPROP_STYLE,STYLE_DASHDOTDOT);
      ObjectSet(name_level,OBJPROP_COLOR,LevelColor);
     }
   double obj_level = NormalizeDouble(ObjectGet(name_level,OBJPROP_PRICE1),4);
   if(Level != obj_level)
     {
      Level = MathRound(obj_level);
      ObjectMove(name_level,0,Time[0],Level);
     }
   // if on the chart there are not enough bars the indicator is not calculate
   if(Bars <= ADX_Period) return(0);
   // amount not changed bars after last call of the indicator 
   int counted_bars = IndicatorCounted();
   if(counted_bars < 0) return(-1);
   // last counted bar will be counted
   if(counted_bars > 0) counted_bars--;
   int limit = Bars - counted_bars;
   // get values of ADX
   if(!UseSmooth_ADX)
     {
      for(int i=0; i<limit; i++)
        {
         ADXBuffer[i] = iADX(NULL,0,ADX_Period,ADX_AppPrice,MODE_MAIN,i);
         ADXBufferSmooth[i] = EMPTY_VALUE;
        }
     }
   else
     {
      for(i=0; i<limit; i++) ADXBufferSmooth[i] = iADX(NULL,0,ADX_Period,ADX_AppPrice,MODE_MAIN,i);
      for(i=0; i<limit; i++) ADXBuffer[i] = iMAOnArray(ADXBufferSmooth,0,ADX_MASmooth_Period,0,ADX_MASmooth_Method,i);
     }
   if(!UseSmooth_DI)
     {
      for(i=0; i<limit; i++)
        {
         pDIBuffer[i] = iADX(NULL,0,ADX_Period,ADX_AppPrice,MODE_PLUSDI,i);
         mDIBuffer[i] = iADX(NULL,0,ADX_Period,ADX_AppPrice,MODE_MINUSDI,i);
         pDIBufferSmooth[i] = EMPTY_VALUE;
         mDIBufferSmooth[i] = EMPTY_VALUE;
        }
     }
   else
     {
      for(i=0; i<limit; i++)
        {
         pDIBufferSmooth[i] = iADX(NULL,0,ADX_Period,ADX_AppPrice,MODE_PLUSDI,i);
         mDIBufferSmooth[i] = iADX(NULL,0,ADX_Period,ADX_AppPrice,MODE_MINUSDI,i);
        }
      for(i=0; i<limit; i++)
        {
         pDIBuffer[i] = iMAOnArray(pDIBufferSmooth,0,DI_MASmooth_Period,0,DI_MASmooth_Method,i);
         mDIBuffer[i] = iMAOnArray(mDIBufferSmooth,0,DI_MASmooth_Period,0,DI_MASmooth_Method,i);
        }
     }
   // alert when the line ADX to cross a Level from below to top.
   if(UseSignal_ADX)
     {
      if(prevtime_adx != Time[0])
        {
         if(ADXBuffer[shift+1] <= Level && ADXBuffer[shift] > Level)
           {
            if(pDIBuffer[shift] > mDIBuffer[shift])
              {
               if(ADX_SoundAlert) PlaySound(ADX_SoundFile_Buy);
               if(ADX_EmailAlert) SendMail(StringConcatenate("Alert from indicator \"",short_name,"\""),StringConcatenate("BUY Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),adx_message));
               if(ADX_PopUpAlert) Alert(StringConcatenate("Indicator \"",short_name,"\":\nBUY Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),"\n",adx_message));
               if(ADX_DrawVertLine)
                 {
                  string name = StringConcatenate("adx_vline BUY ",Symbol()," ",TimeToStr(Time[0]));
                  ObjectCreate(name,OBJ_VLINE,0,Time[0],0);
                  ObjectSet(name,OBJPROP_COLOR,ADX_VLineColor_Buy); 
                 }
              }
            else
              {
               if(ADX_SoundAlert) PlaySound(ADX_SoundFile_Sell);
               if(ADX_EmailAlert) SendMail(StringConcatenate("Alert from indicator \"",short_name,"\""),StringConcatenate("SELL Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),adx_message));
               if(ADX_PopUpAlert) Alert(StringConcatenate("Indicator \"",short_name,"\":\nSELL Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),"\n",adx_message));
               if(ADX_DrawVertLine)
                 {
                  name = StringConcatenate("adx_vline SELL ",Symbol()," ",TimeToStr(Time[0]));
                  ObjectCreate(name,OBJ_VLINE,0,Time[0],0);
                  ObjectSet(name,OBJPROP_COLOR,ADX_VLineColor_Sell);
                 }
              }
            prevtime_adx = Time[0];
           }
        }
     }
   // alert when lines +DI and -DI are crossed
   if(UseSignal_DI)
     {
      if(prevtime_di != Time[0])
        {
         if(pDIBuffer[shift+1] <= mDIBuffer[shift+1] && pDIBuffer[shift] > mDIBuffer[shift])
           {
            if(DI_SoundAlert) PlaySound(DI_SoundFile_Buy);
            if(DI_EmailAlert) SendMail(StringConcatenate("Alert from indicator \"",short_name,"\""),StringConcatenate("BUY Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),pdi_message));
            if(DI_PopUpAlert) Alert(StringConcatenate("Indicator \"",short_name,"\":\nBUY Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),"\n",pdi_message));
            if(DI_DrawVertLine)
              {
               name = StringConcatenate("adx_vline BUY ",Symbol()," ",TimeToStr(Time[0]));
               ObjectCreate(name,OBJ_VLINE,0,Time[0],0);
               ObjectSet(name,OBJPROP_COLOR,DI_VLineColor_Buy); 
              }
            prevtime_di = Time[0];
           }
         if(pDIBuffer[shift+1] >= mDIBuffer[shift+1] && pDIBuffer[shift] < mDIBuffer[shift])
           {
            if(DI_SoundAlert) PlaySound(DI_SoundFile_Sell);
            if(DI_EmailAlert) SendMail(StringConcatenate("Alert from indicator \"",short_name,"\""),StringConcatenate("SELL Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),mdi_message));
            if(DI_PopUpAlert) Alert(StringConcatenate("Indicator \"",short_name,"\":\nSELL Signal on ",Symbol()," at ",TimeToStr(TimeCurrent()),"\n",mdi_message));
            if(DI_DrawVertLine)
              {
               name = StringConcatenate("adx_vline SELL ",Symbol()," ",TimeToStr(Time[0]));
               ObjectCreate(name,OBJ_VLINE,0,Time[0],0);
               ObjectSet(name,OBJPROP_COLOR,DI_VLineColor_Sell); 
              }
            prevtime_di = Time[0];
           }
        }
     }
   // finish of iteration
   return(0);
  }
  
//+------------------------------------------------------------------+